from numpy import ndarray
from numpy.random import uniform
from aidev.utilities.types import Sampling
from scipy.stats import qmc


class Random(Sampling):
    """
    Class for random design space sampling.
    """

    def __init__(self, definitions) -> None:
        """

        :param definitions: Collective targets and parameters information in the form of FeatureDefinition class
        :type definitions: FeatureDefinition
        """
        super().__init__(definitions)

    def _method(self, n_samples: int = 50) -> ndarray:
        """

        Method defining the sampling strategy.

        :param n_samples: Number of samples to generate in the DoE.
        :type n_samples: int
        :return: The samples.
        :rtype: ndarray
        """
        samples = uniform(self.lbs, self.ubs, (n_samples, self.nvar))
        return samples


class Latin(Sampling):
    """
    Class for Latin design space sampling.
    """

    def __init__(self, definitions) -> None:
        """

        :param definitions: Collective targets and parameters information in the form of FeatureDefinition class
        :type definitions: FeatureDefinition
        """
        super().__init__(definitions)

    def _method(self, n_samples: int = 50) -> ndarray:
        """

        Method defining the sampling strategy.

        :param n_samples: Number of samples to generate in the DoE.
        :type n_samples: int
        :return: The samples.
        :rtype: ndarray
        """
        sampler = qmc.LatinHypercube(d=self.nvar)
        samp = sampler.random(n=n_samples)
        samples = qmc.scale(samp, self.lbs, self.ubs)
        return samples
